// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"
#include "display/RenderManager.hpp"
#include "scene/Scene.hpp"
#include "scene/components/Components.hpp"
#include "scene/components/ui/UICanvas.hpp"
#include "ui/UIComponent.hpp"

namespace Dawn {
  class RenderPipeline {
    public:
      RenderManager *renderManager;

      /**
       * Constructs a new RenderPipeline. Render Pipelines are my attempt to
       * create both a flexible, but standard way to allow the individual games
       * to decide how they want to render the common scene-item models.
       * 
       * @param renderManager Parent render manager this pipeline belongs to.
       */
      RenderPipeline(RenderManager *renderManager);

      /**
       * Initialize the render pipeline.
       */
      virtual void init();
      
      /**
       * Renders the games' currently active scene, and all of its' cameras.
       */
      virtual void render();

      /**
       * Render a specific scene, usually just called for the currently active
       * scene, but in future this could include sub-scenes.
       * 
       * @param scene Scene to render.
       */
      virtual void renderScene(Scene *scene);

      /**
       * Render a specific camera on a specific scene.
       * 
       * @param scene Scene to render.
       * @param camera Camera within the scene to render.
       */
      virtual void renderSceneCamera(Scene *scene, Camera *camera);

      /**
       * Renders a UI Canvas to the back buffer.
       * 
       * @param scene Scene for the UI canvas.
       * @param camera Main backbuffer camera for the canvas.
       * @param canvas Canvas to render.
       */
      virtual void renderUI(
        Scene *scene,
        Camera *camera,
        UICanvas *canvas
      );

      /**
       * Cleanup a render pipeline that has been initialized.
       */
      virtual ~RenderPipeline();
  };
}