// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItemComponent.hpp"
#include "display/shader/Shader.hpp"

namespace Dawn {
  class Shader;
  
  class Material : public SceneItemComponent {
    private:
      Shader *shader = nullptr;

      /**
       * Internal method that will be invoked to go through and update all of
       * the shader parameters whenever the shader is swapped out.
       */
      void updateShaderParameters();

    public:
      std::map<shaderparameter_t, enum ShaderParameterType> parameters;
      std::map<shaderparameter_t, struct Color> colorValues;
      std::map<shaderparameter_t, bool_t> boolValues;
      std::map<shaderparameter_t, glm::mat4> matrixValues;
      std::map<shaderparameter_t, glm::vec3> vec3Values;
      std::map<shaderparameter_t, Texture*> textureValues;
      std::map<shaderparameter_t, float_t> floatValues;

      /**
       * Material component constructor. 
       * 
       * @param item Scene Item this component belongs to.
       */
      Material(SceneItem *item);

      /**
       * Return the shader this material is currently using.
       * 
       * @return Shader pointer to the currently bound shader.
       */
      Shader * getShader();

      /**
       * Sets the shader for the material to use. This will also clear and
       * update all of the parameters from the shaders' default parameter list.
       * 
       * @param shader Shader to set.
       */
      void setShader(Shader * shader);

      /**
       * Protected method that can be called, likely by the render pipeline, to
       * set and update all of the shader parameters from this material on to
       * the shader.
       * 
       * This method assumes that the shader has already been bound.
       */
      void setShaderParameters();
  };
}