// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Prefab.hpp"
#include "ui/UIBorder.hpp"
#include "asset/assets/TextureAsset.hpp"
#include "game/DawnGame.hpp"

#define UI_BORDER_TEXTURE_ASSET "texture_test"

namespace Dawn {
  class UIBorderPrefab : public UIPrefab<UIBorder> {
    protected:
      static std::vector<Asset*> getAssets(AssetManager *man) override {
        return std::vector<Asset*>{
          man->get<TextureAsset>(UI_BORDER_TEXTURE_ASSET)
        };
      }

      static SceneItem * uiCreate(UICanvas *canvas) override {
        auto border = canvas->addElement<UIBorder>();
        UIPrefab::uiApply(border);
        return border;
      }

    public:
      void uiApply(UIBorder *border) {
        border->texture = &border
          ->getGame()
          ->assetManager.get<TextureAsset>(UI_BORDER_TEXTURE_ASSET)->texture
        ;
        border->setBorderSize(glm::vec2(16, 16));
      }
  }
}